﻿#region Imported Namespaces

using System;
using Christian.Helle.PocketNews.PocketPC.Utilities;

#endregion

namespace Christian.Helle.PocketNews.PocketPC.Diagnostics
{
    /// <summary>
    /// Utility for writing to a log file. Default log file is [Working Directory]\Error.txt
    /// </summary>
    public sealed class ErrorLogFile : LogFile
    {
        /// <summary>
        /// Creates an instance of <see cref="ErrorLogFile"/>
        /// </summary>
        public ErrorLogFile()
        {
            Filename = GetDefaultFile();
        }

        /// <summary>
        /// Singleton implementation of <see cref="ErrorLogFile"/>
        /// </summary>
        public static ErrorLogFile Instance
        {
            get { return Singleton<ErrorLogFile>.GetInstance(); }
        }

        /// <summary>
        /// Retrieves the default log file to use
        /// </summary>
        /// <returns></returns>
        protected override string GetDefaultFile()
        {
            return LogFileRepository.GetLogFile("Error");
        }

        /// <summary>
        /// Writes an exception to the log file
        /// </summary>
        /// <param name="source">Source of the Exception</param>
        /// <param name="message">Exception Message</param>
        /// <param name="stacktrace">Exception or the Environments Stack Trace</param>
        public void AppendError(string source, string message, string stacktrace)
        {
            try
            {
                WriteLine(
                    string.Format(
                        @"-=-=-=-=-=--=-=--=-=-=-=-=--=-=--=-=-=-=-=--=-=--=-=-=-=-=--=-=-{3}{0}:{3}{1}{3}{3}STACK TRACE:{3}{2}{3}{3}{3}",
                        DateTime.Now,
                        message,
                        stacktrace,
                        "\r\n"));
            }
            catch (Exception e)
            {
                TraceException(e);
            }
        }

        /// <summary>
        /// Writes an exception to the log file
        /// </summary>
        /// <param name="source">Source of the Exception</param>
        /// <param name="message">Exception Message</param>
        /// <param name="innerexception">InnerExceptions Message</param>
        /// <param name="stacktrace">Exception or the Environments Stack Trace</param>
        public void AppendError(string source, string message, string innerexception,
                                string stacktrace)
        {
            try
            {
                if (string.IsNullOrEmpty(innerexception))
                {
                    AppendError(source, message, stacktrace);
                }
                else
                {
                    WriteLine(
                        string.Format(
                            "-=-=-=-=-=--=-=--=-=-=-=-=--=-=--=-=-=-=-=--=-=--=-=-=-=-=--=-=-{4}{0}:\t{1}{4}INNER EXCEPTION:\t{2}{4}STACK TRACE:{4}{3}{4}{4}{4}",
                            DateTime.Now,
                            message,
                            innerexception,
                            stacktrace,
                            "\r\n"));
                }
            }
            catch (Exception e)
            {
                TraceException(e);
            }
        }

        /// <summary>
        /// Writes an exception to the log file
        /// </summary>
        /// <param name="exception">Instance of the exception object</param>
        public void AppendException(Exception exception)
        {
            if (exception == null)
                throw new ArgumentNullException("exception");

            try
            {
                WriteLine(
                    string.Format(
                        "-=-=-=-=-=--=-=--=-=-=-=-=--=-=--=-=-=-=-=--=-=--=-=-=-=-=--=-=-{2}{0}{1}{0}{2}{2}{2}",
                        DateTime.Now,
                        exception,
                        "\r\n"));
            }
            catch (Exception e)
            {
                TraceException(e);
            }
        }

        /// <summary>
        /// Writes an exception to the log file
        /// </summary>
        /// <param name="source">Source of the exception</param>
        /// <param name="exception">Instance of the exception object</param>
        public void AppendException(string source, Exception exception)
        {
            if (exception == null)
                throw new ArgumentNullException("exception");

#if !PocketPC
            if (!string.IsNullOrEmpty(source))
                source = exception.Source;
#endif

            if (exception.InnerException != null)
                AppendError(source, exception.Message, exception.InnerException.Message, exception.StackTrace);
            else
                AppendError(source, exception.Message, exception.StackTrace);
        }
    }
}